@@ -1,8 +1,10 @@
+from __future__ import annotations
 import time
 import threading
 import logging
 from ...utils.time_service import TimeService
-from ...geonet.service_access_point import Area, GeoBroadcastHST, HeaderType
+from ..ca_basic_service.cam_transmission_management import VehicleData
+from ...geonet.service_access_point import Area, GeoBroadcastHST, HeaderType, PacketTransportType
 from .denm_coder import DENMCoder
 from ...btp.router import Router as BTPRouter
 from ...btp.service_access_point import BTPDataRequest, CommonNH, CommunicationProfile
@@ -11,23 +13,6 @@ from ...applications.road_hazard_signalling_service.service_access_point import
 )
 
 
-class VehicleData:
-    """
-    Class that stores the vehicle data.
-
-    Attributes
-    ----------
-    station_id : int
-        Station ID as specified in ETSI TS 103 831 V2.1.1 (2022-11).
-    station_type : int
-        Station Type as specified in ETSI TS 103 831 V2.1.1 (2022-11).
-    """
-
-    def __init__(self) -> None:
-        self.station_id = 0
-        self.station_type = 0
-
-
 class DecentralizedEnvironmentalNotificationMessage:
     """
     Decentralized Environmental Notification Message class.
@@ -210,8 +195,7 @@ class DENMTransmissionManagement:
         self.btp_router: BTPRouter = btp_router
         self.vehicle_data = vehicle_data
         self.denm_coder = denm_coder
-        self.crw_denm = DecentralizedEnvironmentalNotificationMessage()
-        self.new_denm = DecentralizedEnvironmentalNotificationMessage()
+        self.sequence_number = 0
 
     def request_denm_sending(self, denm_request: DENRequest) -> None:
         """
@@ -225,10 +209,10 @@ class DENMTransmissionManagement:
         """
         Request to send a single DENM message with the Collision Risk Warning data.
         """
-        self.crw_denm = DecentralizedEnvironmentalNotificationMessage()
-        self.crw_denm.fullfill_with_vehicle_data(self.vehicle_data)
-        self.crw_denm.fullfill_with_collision_risk_warning(denm_request)
-        self.transmit_denm(self.crw_denm)
+        crw_denm = DecentralizedEnvironmentalNotificationMessage()
+        crw_denm.fullfill_with_vehicle_data(self.vehicle_data)
+        crw_denm.fullfill_with_collision_risk_warning(denm_request)
+        self.transmit_denm(crw_denm)
 
     def trigger_denm_messages(self, denm_request: DENRequest) -> None:
         """
@@ -240,13 +224,11 @@ class DENMTransmissionManagement:
             DENM Request object.
         """
         transmission_time = 0
-
         while transmission_time < denm_request.time_period:
-            self.new_denm = DecentralizedEnvironmentalNotificationMessage()
-            self.new_denm.fullfill_with_vehicle_data(self.vehicle_data)
-            self.new_denm.fullfill_with_denrequest(denm_request)
-
-            self.transmit_denm(self.new_denm)
+            new_denm = DecentralizedEnvironmentalNotificationMessage()
+            new_denm.fullfill_with_vehicle_data(self.vehicle_data)
+            new_denm.fullfill_with_denrequest(denm_request)
+            self.transmit_denm(new_denm)
             time.sleep(denm_request.denm_interval / 1000)
             transmission_time += denm_request.denm_interval
 
@@ -261,29 +243,32 @@ class DENMTransmissionManagement:
         denm_to_send : DecentralizedEnvironmentalNotificationMessage
             New DENM message to be sent.
         """
-        request = BTPDataRequest()
-        request.btp_type = CommonNH.BTP_B
-        request.destination_port = 2002
-        request.gn_packet_transport_type.header_type = HeaderType.GEOBROADCAST
-        request.gn_packet_transport_type.header_subtype = (
-            GeoBroadcastHST.GEOBROADCAST_CIRCLE
+        data = self.denm_coder.encode(denm_to_send.denm)
+        request = BTPDataRequest(
+            btp_type=CommonNH.BTP_B,
+            destination_port=2002,
+            gn_packet_transport_type=PacketTransportType(
+                header_subtype=GeoBroadcastHST.GEOBROADCAST_CIRCLE,
+                header_type=HeaderType.GEOBROADCAST,
+            ),
+            gn_area=Area(
+                a=100,
+                b=0,
+                angle=0,
+                latitude=denm_to_send.denm["denm"]["management"][
+                    "eventPosition"
+                ]["latitude"],
+                longitude=denm_to_send.denm["denm"]["management"][
+                    "eventPosition"
+                ]["longitude"],
+            ),
+            communication_profile=CommunicationProfile.UNSPECIFIED,
+            data=data,
+            length=len(data),
         )
-        request.gn_area = Area()
-        request.gn_area.a = 100
-        request.gn_area.b = 0
-        request.gn_area.angle = 0
-        request.gn_area.latitude = denm_to_send.denm["denm"]["management"][
-            "eventPosition"
-        ]["latitude"]
-        request.gn_area.longitude = denm_to_send.denm["denm"]["management"][
-            "eventPosition"
-        ]["longitude"]
-        request.communication_profile = CommunicationProfile.UNSPECIFIED
-        request.data = self.denm_coder.encode(denm_to_send.denm)
-        request.length = len(request.data)
         self.btp_router.btp_data_request(request)
         self.logging.debug(
-            "Recieved DENM with timestamp: %s, station_id: %s",
+            "Sent DENM with timestamp: %s, station_id: %s",
             denm_to_send.denm["denm"]["management"]["referenceTime"],
             denm_to_send.denm["header"]["stationId"],
         )
