@@ -1,6 +1,7 @@
 from __future__ import annotations
 from collections.abc import Callable
 from enum import Enum
+from threading import Lock
 import math
 from ..linklayer.exceptions import (
     SendingException,
@@ -71,7 +72,7 @@ class Router:
 
     """
 
-    def __init__(self, mib: MIB, sign_service: SignService = None) -> None:
+    def __init__(self, mib: MIB, sign_service: SignService | None = None) -> None:
         """
         Initialize the router.
 
@@ -81,14 +82,14 @@ class Router:
             MIB to use.
         """
         self.mib = mib
-        self.gn_address = GNAddress()
-        self.setup_gn_address()
+        self.ego_position_vector_lock = Lock()
         self.ego_position_vector = LongPositionVector()
-        self.ego_position_vector.set_gn_addr(self.gn_address)
+        self.setup_gn_address()
         self.link_layer: LinkLayer | None = None
         self.location_table = LocationTable(mib)
-        self.sign_service: SignService = sign_service
+        self.sign_service: SignService | None = sign_service
         self.indication_callback = None
+        self.sequence_number_lock = Lock()
         self.sequence_number = 0
 
     def get_sequence_number(self) -> int:
@@ -100,8 +101,9 @@ class Router:
         int
             Current sequence number.
         """
-        self.sequence_number = (self.sequence_number + 1) % (2**16 - 1)
-        return self.sequence_number
+        with self.sequence_number_lock:
+            self.sequence_number = (self.sequence_number + 1) % (2**16 - 1)
+            return self.sequence_number
 
     def register_indication_callback(
         self, callback: Callable[[GNDataIndication], None]
@@ -131,7 +133,8 @@ class Router:
                 "Managed GN address configuration is not implemented."
             )
         elif self.mib.itsGnLocalGnAddrConfMethod == LocalGnAddrConfMethod.AUTO:
-            self.gn_address = self.mib.itsGnLocalGnAddr
+            self.ego_position_vector = self.ego_position_vector.set_gn_addr(
+                self.mib.itsGnLocalGnAddr)
         elif self.mib.itsGnLocalGnAddrConfMethod == LocalGnAddrConfMethod.ANONYMOUS:
             raise NotImplementedError(
                 "Anonymous GN address configuration is not implemented."
@@ -146,20 +149,14 @@ class Router:
         request : GNDataRequest
             GNDataRequest to handle.
         """
-        basic_header = BasicHeader()
-        basic_header.initialize_with_mib(self.mib)
-        basic_header.set_rhl(1)
-        common_header = CommonHeader()
-        common_header.initialize_with_request(request)
+        basic_header = BasicHeader.initialize_with_mib_and_rhl(self.mib, 1)
+        common_header = CommonHeader.initialize_with_request(request)
         long_position_vector = self.ego_position_vector
         media_dependant_data = b"\x00\x00\x00\x00"
         packet = b""
         if request.security_profile == SecurityProfile.COOPERATIVE_AWARENESS_MESSAGE:
             if self.sign_service is None:
                 raise NotImplementedError("Security profile not implemented")
-            common_header = CommonHeader()
-            common_header.initialize_with_request(request)
-            long_position_vector = self.ego_position_vector
             media_dependant_data = b"\x00\x00\x00\x00"
             tbs_packet = (
                 common_header.encode_to_bytes()
@@ -176,7 +173,7 @@ class Router:
             )
             sign_confirm: SNSIGNConfirm = self.sign_service.sign_cam(
                 sign_request)
-            basic_header.set_nh(BasicNH.SECURED_PACKET)
+            basic_header = basic_header.set_nh(BasicNH.SECURED_PACKET)
             packet = basic_header.encode_to_bytes() + sign_confirm.sec_message
 
         else:
@@ -187,19 +184,16 @@ class Router:
                 + media_dependant_data
                 + request.data
             )
-        confirm = GNDataConfirm()
 
         try:
             if self.link_layer:
                 self.link_layer.send(packet)
         except PacketTooLongException:
-            confirm.result_code = ResultCode.MAXIMUM_LENGTH_EXCEEDED
+            return GNDataConfirm(result_code=ResultCode.MAXIMUM_LENGTH_EXCEEDED)
         except SendingException:
-            confirm.result_code = ResultCode.UNSPECIFIED
+            return GNDataConfirm(result_code=ResultCode.UNSPECIFIED)
 
-        confirm.result_code = ResultCode.ACCEPTED
-
-        return confirm
+        return GNDataConfirm(result_code=ResultCode.ACCEPTED)
 
     @staticmethod
     def calculate_distance(
@@ -326,22 +320,26 @@ class Router:
         packet : bytes
             Packet to forward. (Without headers)
         """
-        confirm = GNDataConfirm()
         # TODO: Location Service (LS) packet buffers (step 8)
-        basic_header.set_rhl(basic_header.rhl - 1)
+        basic_header = basic_header.set_rhl(basic_header.rhl - 1)
         # 10) if no neighbour exists, i.e. the LocT does not contain a LocTE with the IS_NEIGHBOUR flag set to TRUE,
         # and SCF for the traffic class in the TC field of the Common Header is set, buffer the GBC packet in the BC
         # forwarding packet buffer and omit the execution of further steps;
         if len(self.location_table.get_neighbours()) > 0 or not common_header.tc.scf:
             # 11) execute the forwarding algorithm procedures (starting with annex D);
-            request = GNDataRequest()
-            request.area = Area()
-            request.area.latitude = gbc_extended_header.latitude
-            request.area.longitude = gbc_extended_header.longitude
-            request.area.a = gbc_extended_header.a
-            request.area.b = gbc_extended_header.b
-            request.area.angle = gbc_extended_header.angle
-            request.packet_transport_type.header_subtype = common_header.hst
+            area = Area(
+                latitude=gbc_extended_header.latitude,
+                longitude=gbc_extended_header.longitude,
+                a=gbc_extended_header.a,
+                b=gbc_extended_header.b,
+                angle=gbc_extended_header.angle,
+            )
+            packet_transport_type = PacketTransportType(
+                header_type=common_header.ht,
+                header_subtype=common_header.hst,
+            )
+            request = GNDataRequest(
+                area=area, packet_transport_type=packet_transport_type)
             algorithm = self.gn_forwarding_algorithm_selection(request)
             # 12) if the return value of the forwarding algorithm is 0 (packet is buffered in a forwarding packet
             # buffer) or -1 (packet is discarded), omit the execution of further steps;
@@ -349,7 +347,7 @@ class Router:
                 # TODO: step 13
                 # 14) pass the GN-PDU to the LL protocol entity via the IN interface and set the destination
                 # address to the LL address of the next hop LL_ADDR_NH.
-                packet: bytes = (
+                final_packet: bytes = (
                     basic_header.encode_to_bytes()
                     + common_header.encode_to_bytes()
                     + gbc_extended_header.encode()
@@ -357,15 +355,15 @@ class Router:
                 )
                 try:
                     if self.link_layer:
-                        self.link_layer.send(packet)
+                        self.link_layer.send(final_packet)
                 except PacketTooLongException:
-                    confirm.result_code = ResultCode.MAXIMUM_LENGTH_EXCEEDED
+                    return GNDataConfirm(
+                        result_code=ResultCode.MAXIMUM_LENGTH_EXCEEDED)
                 except SendingException:
-                    confirm.result_code = ResultCode.UNSPECIFIED
+                    return GNDataConfirm(result_code=ResultCode.UNSPECIFIED)
 
-                confirm.result_code = ResultCode.ACCEPTED
         else:
-            packet: bytes = (
+            final_packet: bytes = (
                 basic_header.encode_to_bytes()
                 + common_header.encode_to_bytes()
                 + gbc_extended_header.encode()
@@ -373,15 +371,13 @@ class Router:
             )
             try:
                 if self.link_layer:
-                    self.link_layer.send(packet)
+                    self.link_layer.send(final_packet)
             except PacketTooLongException:
-                confirm.result_code = ResultCode.MAXIMUM_LENGTH_EXCEEDED
+                return GNDataConfirm(
+                    result_code=ResultCode.MAXIMUM_LENGTH_EXCEEDED)
             except SendingException:
-                confirm.result_code = ResultCode.UNSPECIFIED
-
-            confirm.result_code = ResultCode.ACCEPTED
-
-        return confirm
+                return GNDataConfirm(result_code=ResultCode.UNSPECIFIED)
+        return GNDataConfirm(result_code=ResultCode.ACCEPTED)
 
     def gn_data_request_gbc(self, request: GNDataRequest) -> GNDataConfirm:
         """
@@ -392,22 +388,14 @@ class Router:
         request : GNDataRequest
             GNDataRequest to handle.
         """
-        confirm = GNDataConfirm()
-
         # 1) create a GN-PDU with the T/GN6-SDU as payload and a GBC packet header (clause 9.8.5):
         #   a) set the fields of the Basic Header (clause 10.3.2);
-        basic_header = BasicHeader()
-        basic_header.initialize_with_mib(self.mib)
+        basic_header = BasicHeader.initialize_with_mib(self.mib)
         #   b) set the fields of the Common Header (clause 10.3.4);
-        common_header = CommonHeader()
-        common_header.initialize_with_request(request)
-        common_header.ht = HeaderType.GEOBROADCAST
-        common_header.hst = request.packet_transport_type.header_subtype
+        common_header = CommonHeader.initialize_with_request(request)
         #   c) set the fields of the GBC Extended Header (table 36);
-        geo_broadcast_extended_header = GBCExtendedHeader()
-        geo_broadcast_extended_header.initialize_with_request(request)
-        geo_broadcast_extended_header.sn = self.get_sequence_number()
-        geo_broadcast_extended_header.so_pv = self.ego_position_vector
+        geo_broadcast_extended_header = GBCExtendedHeader.initialize_with_request_sequence_number_ego_pv(
+            request, self.get_sequence_number(), self.ego_position_vector)
         # 2) if no neighbour exists, i.e. the LocT does not contain a LocTE with the IS_NEIGHBOUR flag set to TRUE,
         # and SCF for the traffic class in the service primitive GN-DATA.request parameter Traffic class is enabled,
         # then buffer the GBC packet in the BC forwarding packet buffer and omit the execution of further steps;
@@ -433,11 +421,10 @@ class Router:
                     if self.link_layer:
                         self.link_layer.send(packet)
                 except PacketTooLongException:
-                    confirm.result_code = ResultCode.MAXIMUM_LENGTH_EXCEEDED
+                    return GNDataConfirm(result_code=ResultCode.MAXIMUM_LENGTH_EXCEEDED)
                 except SendingException:
-                    confirm.result_code = ResultCode.UNSPECIFIED
+                    return GNDataConfirm(result_code=ResultCode.UNSPECIFIED)
 
-                confirm.result_code = ResultCode.ACCEPTED
         else:
             packet: bytes = (
                 basic_header.encode_to_bytes()
@@ -449,13 +436,11 @@ class Router:
                 if self.link_layer:
                     self.link_layer.send(packet)
             except PacketTooLongException:
-                confirm.result_code = ResultCode.MAXIMUM_LENGTH_EXCEEDED
+                return GNDataConfirm(result_code=ResultCode.MAXIMUM_LENGTH_EXCEEDED)
             except SendingException:
-                confirm.result_code = ResultCode.UNSPECIFIED
+                return GNDataConfirm(result_code=ResultCode.UNSPECIFIED)
 
-            confirm.result_code = ResultCode.ACCEPTED
-
-        return confirm
+        return GNDataConfirm(result_code=ResultCode.ACCEPTED)
 
     def gn_data_request(self, request: GNDataRequest) -> GNDataConfirm:
         """
@@ -497,7 +482,6 @@ class Router:
             CommonHeader of the packet.
         """
         # ETSI EN 302 636-4-1 V1.4.1 (2020-01). Section
-        indication = GNDataIndication()
         try:
             long_position_vector = LongPositionVector()
             long_position_vector.decode(packet[0:24])
@@ -505,11 +489,13 @@ class Router:
             # Ignore Media Dependant Data
             packet = packet[4:]
             self.location_table.new_shb_packet(long_position_vector, packet)
-            indication.upper_protocol_entity = common_header.nh
-            indication.source_position_vector = long_position_vector
-            indication.traffic_class = common_header.tc
-            indication.length = len(packet)
-            indication.data = packet
+            return GNDataIndication(
+                upper_protocol_entity=common_header.nh,
+                source_position_vector=long_position_vector,
+                traffic_class=common_header.tc,
+                length=len(packet),
+                data=packet
+            )
         except DADException:
             print("Duplicate Address Detected!")
         except IncongruentTimestampException:
@@ -518,7 +504,7 @@ class Router:
             print("Packet is duplicated")
         except DecodeError as e:
             print(str(e))
-        return indication
+        return GNDataIndication()
 
     def gn_data_indicate_gbc(
         self, packet: bytes, common_header: CommonHeader
@@ -533,18 +519,17 @@ class Router:
         common_header : CommonHeader
             CommonHeader of the packet.
         """
-        indication = GNDataIndication()
-        gbc_extended_header = GBCExtendedHeader()
-        gbc_extended_header.decode(packet[0:44])
+        gbc_extended_header = GBCExtendedHeader.decode(packet[0:44])
         packet = packet[44:]
-        area = Area()
-        area.a = gbc_extended_header.a
-        area.b = gbc_extended_header.b
-        area.latitude = gbc_extended_header.latitude
-        area.longitude = gbc_extended_header.longitude
-        area.angle = gbc_extended_header.angle
+        area = Area(
+            a=gbc_extended_header.a,
+            b=gbc_extended_header.b,
+            latitude=gbc_extended_header.latitude,
+            longitude=gbc_extended_header.longitude,
+            angle=gbc_extended_header.angle
+        )
         area_f = self.gn_geometric_function_f(
-            common_header.hst,
+            common_header.hst,  # type: ignore
             area,
             self.ego_position_vector.latitude,
             self.ego_position_vector.longitude,
@@ -566,24 +551,25 @@ class Router:
             self.duplicate_address_detection(gbc_extended_header.so_pv.gn_addr)
             self.location_table.new_gbc_packet(gbc_extended_header, packet)
             if area_f >= 0:
-                indication.upper_protocol_entity = common_header.nh
-                indication.packet_transport_type = PacketTransportType()
-                indication.packet_transport_type.header_type = HeaderType.GEOBROADCAST
-                indication.packet_transport_type.header_subtype = common_header.hst
-                indication.source_position_vector = gbc_extended_header.so_pv
-                indication.traffic_class = common_header.tc
                 # TODO: Extend the indication information
-                # indication.length = len(packet[44:])
-                # indication.data = packet[44:]
-                indication.length = len(packet)
-                indication.data = packet
+                return GNDataIndication(
+                    upper_protocol_entity=common_header.nh,
+                    packet_transport_type=PacketTransportType(
+                        header_type=HeaderType.GEOBROADCAST,
+                        header_subtype=common_header.hst
+                    ),
+                    source_position_vector=gbc_extended_header.so_pv,
+                    traffic_class=common_header.tc,
+                    length=len(packet),
+                    data=packet
+                )
         except DADException:
             print("Duplicate Address Detected!")
         except IncongruentTimestampException:
             print("Incongruent Timestamp Detected!")
         except DecodeError as e:
             print(str(e))
-        return indication
+        return GNDataIndication()
 
     def gn_data_indicate(self, packet: bytes) -> None:
         # pylint: disable=no-else-raise, too-many-branches
@@ -604,16 +590,14 @@ class Router:
         indication = GNDataIndication()
         # ETSI EN 302 636-4-1 V1.4.1 (2020-01). Section 10.3.3
         # Decap the common header
-        basic_header = BasicHeader()
-        basic_header.decode_from_bytes(packet[0:4])
+        basic_header = BasicHeader.decode_from_bytes(packet[0:4])
         packet = packet[4:]
         if basic_header.version != self.mib.itsGnProtocolVersion:
             raise NotImplementedError("Version not implemented")
         if basic_header.nh == BasicNH.COMMON_HEADER:
             # ETSI EN 302 636-4-1 V1.4.1 (2020-01). Section 10.3.5
             # Decap the common header
-            common_header = CommonHeader()
-            common_header.decode_from_bytes(packet[0:8])
+            common_header = CommonHeader.decode_from_bytes(packet[0:8])
             packet = packet[8:]
             if basic_header.rhl > self.mib.itsGnDefaultHopLimit:
                 raise DecapError("Hop limit exceeded")
@@ -666,4 +650,6 @@ class Router:
         """
         Refresh the ego position vector.
         """
-        self.ego_position_vector.refresh_with_tpv_data(tpv)
+        with self.ego_position_vector_lock:
+            self.ego_position_vector = self.ego_position_vector.refresh_with_tpv_data(
+                tpv)
